import { configureAxios, request } from '../requests'

// ======================
//         Alerts
// ======================

export function addAlertError(message) {
  return {
    type: 'ADD_ALERT_ERROR',
    payload: {
      message,
    }
  }
}

export function addAlertOk(message) {
  return {
    type: 'ADD_ALERT_OK',
    payload: {
      message,
    }
  }
}

export function dismissAlert() {
  return {
    type: 'DISMISS_ALERT',
  }
}

// ======================
//         Users
// ======================

export function userLogout() {
  return {
    type: 'USER_LOGOUT',
  }
}

export function loginUser(username, password) {
  return request(
    'USER_LOGIN',
    configureAxios().post(
      '/users/login',
      {
        username: username,
        password: password,
      },
    ),
  )
}

export function updateUser(config) {
  return request(
    'USER_UPDATE',
    configureAxios().post('/users/edit', config),
    [
      addAlertOk("User updated"),
    ],
  )
}

export function userSignUp(config) {
  return request(
    'USER_SIGNUP',
    configureAxios().post('/users/signup', config)
  )
}

export function getUserInfos() {
  return request(
    'GET_USER',
    configureAxios().get('/users/details')
  )
}

// ======================
//         Movies
// ======================

export function updateLastMovieFetchUrl(url) {
  return {
    type: 'UPDATE_LAST_MOVIE_FETCH_URL',
    payload: {
      url: url,
    },
  }
}

export function selectMovie(imdbId) {
  return {
    type: 'SELECT_MOVIE',
    imdbId
  }
}

export function getMovieExploreOptions() {
  return request(
    'MOVIE_GET_EXPLORE_OPTIONS',
    configureAxios().get('/movies/explore/options')
  )
}

export function getMovieDetails(imdbId) {
  return request(
    'MOVIE_GET_DETAILS',
    configureAxios().post(`/movies/${imdbId}/refresh`)
  )
}

export function deleteMovie(imdbId, lastFetchUrl) {
  return request(
    'MOVIE_DELETE',
    configureAxios().delete(`/movies/${imdbId}`),
    [
      fetchMovies(lastFetchUrl),
      addAlertOk("Movie deleted"),
    ],
  )
}

export function addMovieToWishlist(imdbId) {
  return request(
    'MOVIE_ADD_TO_WISHLIST',
    configureAxios().post(`/wishlist/movies/${imdbId}`),
    [
      addAlertOk("Movie added to the wishlist"),
      updateMovieWishlistStore(imdbId, true),
    ],
  )
}

export function deleteMovieFromWishlist(imdbId) {
  return request(
    'MOVIE_DELETE_FROM_WISHLIST',
    configureAxios().delete(`/wishlist/movies/${imdbId}`),
    [
      addAlertOk("Movie deleted from the wishlist"),
      updateMovieWishlistStore(imdbId, false),
    ],
  )
}

export function updateMovieWishlistStore(imdbId, wishlisted) {
  return {
    type: 'MOVIE_UPDATE_STORE_WISHLIST',
    payload: {
      imdbId,
      wishlisted,
    }
  }
}

export function fetchMovies(url) {
  return request(
    'MOVIE_LIST_FETCH',
    configureAxios().get(url),
    [
      updateLastMovieFetchUrl(url),
    ]
  )
}

// ======================
//         Shows
// ======================

export function fetchShows(url) {
  return request(
    'SHOW_LIST_FETCH',
    configureAxios().get(url),
    [
      updateLastShowsFetchUrl(url),
    ]
  )
}

export function getShowDetails(imdbId) {
  return request(
    'SHOW_GET_DETAILS',
    configureAxios().post(`/shows/${imdbId}/refresh`)
  )
}


export function getEpisodeDetails(imdbId, season, episode) {
  return request(
    'EPISODE_GET_DETAILS',
    configureAxios().post(`/shows/${imdbId}/seasons/${season}/episodes/${episode}`),
  )
}

export function updateEpisodeDetailsStore(imdbId, season, episode) {
  return {
    type: 'EPISODE_GET_DETAILS',
    payload: {
      imdbId,
      season,
      episode,
    },
  }
}

export function fetchShowDetails(imdbId) {
  return request(
    'SHOW_FETCH_DETAILS',
    configureAxios().get(`/shows/${imdbId}`)
  )
}

export function addShowToWishlist(imdbId, season = null, episode = null) {
  return request(
    'SHOW_ADD_TO_WISHLIST',
    configureAxios().post(`/wishlist/shows/${imdbId}`, {
      season: season,
      episode: episode,
    }),
    [
      addAlertOk("Show added to the wishlist"),
      updateShowWishlistStore(imdbId, true, season, episode),
    ],
  )
}

export function deleteShowFromWishlist(imdbId) {
  return request(
    'SHOW_DELETE_FROM_WISHLIST',
    configureAxios().delete(`/wishlist/shows/${imdbId}`),
    [
      addAlertOk("Show deleted from the wishlist"),
      updateShowWishlistStore(imdbId, false),
    ],
  )
}

export function updateShowWishlistStore(imdbId, wishlisted, season = null, episode = null) {
  return {
    type: 'SHOW_UPDATE_STORE_WISHLIST',
    payload: {
      wishlisted: wishlisted,
      imdbId,
      season,
      episode,
    }
  }
}

export function getShowExploreOptions() {
  return request(
    'SHOW_GET_EXPLORE_OPTIONS',
    configureAxios().get('/shows/explore/options')
  )
}

export function selectShow(imdbId) {
  return {
    type: 'SELECT_SHOW',
    imdbId
  }
}

export function updateLastShowsFetchUrl(url) {
  return {
    type: 'UPDATE_LAST_SHOWS_FETCH_URL',
    payload: {
      url: url,
    },
  }
}

// ======================
//      AddTorrent
// ======================

export function addTorrent(url) {
  return request(
    'ADD_TORRENT',
    configureAxios().post('/torrents', {
      url: url,
    }),
    [
      addAlertOk("Torrent added"),
    ],
  )
}

export function fetchTorrents() {
  return request(
    'TORRENTS_FETCH',
    configureAxios().get('/torrents')
  )
}
